#ifndef DELAUNATOR_ITERATOR_H_INCLUDED
#define DELAUNATOR_ITERATOR_H_INCLUDED



/*
 * LIBRARIES
 */
// LOCAL MODULES
#include "commons.h"
#include "vertex.h"
#include "face.h"
#include "edge.h"
#include "vertex_iterators.h"
#include "face_iterators.h"
#include "edge_iterators.h"


/*
 * DEFINES
 */




/*
 * PREDECLARATIONS
 */
// NB: iterators defined next() function for portability to Python's wraps, 
//      or other language where ++ unary operator don't exist.
//      So, next() and operator++ are equivalent.





#endif
